<?php

namespace App\Http\Controllers\Api;

use App\Firmware;
use App\Http\Controllers\Controller;
use App\Http\Requests\StoreFirmware;
use App\Http\Resources\Firmware as FirmwareResource;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Storage;

class FirmwareController extends Controller
{
    public function index(Request $request)
    {
        if ($request->input('per_page')) return Firmware::paginate($request->input('per_page'));
        
        return ['data' => Firmware::all()];
    }

    public function store(StoreFirmware $request)
    {
        $validatedData = $request->validated();
    
        $firmware = Firmware::create($this->firmwareParams($validatedData));
        
        $this->uploadFirmwareFile($firmware, $validatedData['file_data']);
        
        return ['data' => new FirmwareResource($firmware)];
    }

    public function show(Firmware $firmware)
    {
        return ['data' => $firmware];
    }

    public function update(StoreFirmware $request, Firmware $firmware)
    {
        $validatedData = $request->validated();
        
        $this->deleteFirmwareFile($firmware);
    
        $firmware->update($this->firmwareParams($request->validated()));
        
        $this->uploadFirmwareFile($firmware, $validatedData['file_data']);

        return ['data' => new FirmwareResource($firmware)];
    }

    public function destroy(Firmware $firmware)
    {
        $firmware->delete();
        
        $this->deleteFirmwareFile($firmware);

        return response()->json([], 204);
    }
    
    private function firmwareParams(&$validatedData)
    {
        $validatedData['file_data'] = $validatedData['file'];
        $validatedData['file'] = $validatedData['file']->getClientOriginalName();
        
        return $validatedData;
    }
    
    private function uploadFirmwareFile($firmware, $file)
    {
        Storage::disk('firmwares')->putFileAs($this->firmwarePath($firmware), $file, $firmware->file);
    }
    
    private function deleteFirmwareFile($firmware)
    {
        Storage::disk('firmwares')->delete($this->firmwarePath($firmware).$firmware->file);
    }
    
    private function firmwarePath($firmware)
    {
        return '/'.strtolower($firmware->model->brand->name).'/'
            .strtolower($firmware->model->model).'/';
    }
}
