<?php

namespace App\Http\Controllers\Api;

use App\Setting;
use App\Http\Requests\StoreSetting;
use App\Http\Controllers\Controller;
use Illuminate\Http\Request;

class SettingController extends Controller
{
    public function index(Request $request)
    {
        if ($request->input('per_page')) return Setting::paginate($request->input('per_page'));
        
        return ['data' => Setting::getIndex($request->header('tenant'))];
    }
    
    public function store(StoreSetting $request)
    {
        return ['data' => Setting::create($request->validated())];
    }

    public function show(Setting $setting)
    {
        return ['data' => $setting];
    }

    public function update(StoreSetting $request, Setting $setting)
    {
        $setting->update($request->validated());
        
        return ['data' => $setting];
    }

    public function destroy($id)
    {
        $setting = Setting::with('templates')->findOrFail($id);
    
        if ($setting->templates()->count() > 0) return response()->json(['message' => __('messages.settings.cannot_delete')], 422);
    
        $setting->delete();
        
        return response()->json([], 204);
    }
    
    public function verify($id)
    {
        $setting = Setting::with('templates')->findOrFail($id);
        
        return ['data' => $setting->templates()->select('id', 'name')->get()];
    }
}
